import os
import pwd
import re
import subprocess
import tempfile
from stat import S_IRUSR, S_IRGRP


class NoSuchUser(Exception):
    def __init__(self, user):
        message = 'No such user (%s)' % user
        Exception.__init__(self, 'No such user (%s)' % (user,))


class UnableToReadFile(Exception):
    def __init__(self):
        Exception.__init__(self, 'Cannot read sudoers file')


class UnableToWriteFile(Exception):
    def __init__(self):
        Exception.__init__(self, 'Cannot modify sudoers file')


ALIAS_LVECTL_CMDS = [ "/bin/ps", "/bin/grep", "/sbin/service", "/usr/bin/getcontrolpaneluserspackages",
               "/usr/sbin/lvectl", "/usr/local/directadmin/plugins/new_lvemanager/admin/GetDomains" ]
ALIAS_SELECTOR_CMDS = [ "/usr/bin/cl-selector", "/usr/bin/piniset", "/usr/sbin/lveps" ]

DEFAULTS_REQUIRETTY = 'Defaults:%s !requiretty'


class Clsudo(object):
    """
    Adds CloudLinux users to sudoers file
    """
    filepath = '/etc/sudoers'
    temp_dir = '/etc'
    temp_prefix = 'lve_sudoers_'

    def add_user(user):
        """
        Adds username to sudoers file
        """
        Clsudo._check_user(user)
        Clsudo._get_contents(user)

        if not Clsudo.has_alias:
            Clsudo.sudoers_list.append ( 'Cmnd_Alias LVECTL_CMDS = ' + ", ".join( ALIAS_LVECTL_CMDS ) )
        if not Clsudo.has_selector_alias:
            Clsudo.sudoers_list.append('Cmnd_Alias SELECTOR_CMDS = ' + ", ".join( ALIAS_SELECTOR_CMDS ))
        if not Clsudo.has_rights:
            Clsudo.sudoers_list.append('%s ALL=NOPASSWD: LVECTL_CMDS' % (user,))
        if not Clsudo.has_selector_rights:
            Clsudo.sudoers_list.append('%s ALL=NOPASSWD: SELECTOR_CMDS' % (user,))
        if not Clsudo.has_action:
            Clsudo.sudoers_list.append(DEFAULTS_REQUIRETTY % (user,))
        Clsudo._write_contents()
    add_user = staticmethod(add_user)
    
    def add_cagefs_user(user):
        """
        Adds username to sudoers file
        """
        Clsudo._check_user(user)
        Clsudo._get_contents(user)
        if not Clsudo.has_cagefs_alias:
            Clsudo.sudoers_list.append('Cmnd_Alias CAGEFS_CMDS = /usr/sbin/cagefsctl, '
                                       '/bin/ps, /bin/grep, /sbin/service')
        if not Clsudo.has_cagefs_rights:
            Clsudo.sudoers_list.append('%s ALL=NOPASSWD: CAGEFS_CMDS' % (user,))
        if not Clsudo.has_action:
            Clsudo.sudoers_list.append(DEFAULTS_REQUIRETTY % (user,))
        Clsudo._write_contents()
    add_cagefs_user = staticmethod(add_cagefs_user)

    def remove_user(user):
        """
         Removes username from sudoers file
        """
        try:
            f = open(Clsudo.filepath)
            Clsudo.sudoers_list = f.read().splitlines()
            f.close()
            idx = 0
            removed = False
            while idx < len(Clsudo.sudoers_list):
                line = Clsudo.sudoers_list[idx]
                if (('%s ALL=NOPASSWD:' % (user,)) in line) or ((DEFAULTS_REQUIRETTY % (user,))in line):
                    Clsudo.sudoers_list.remove(line)
                    removed = True
                    continue
                idx += 1
            if removed:
                Clsudo._write_contents()
        except (IOError, OSError):
            raise UnableToReadFile()
    remove_user = staticmethod(remove_user)

    def update_user(user):
        """
        updates username in sudoers file
        """
        # Check user presence in system
        Clsudo._check_user(user)
        Clsudo._get_contents(user)
        for idx in range(len(Clsudo.sudoers_list)):
            s = Clsudo.sudoers_list[idx]
            if "Cmnd_Alias LVECTL_CMDS" in s:
                # parse commandlist
                s = s.replace("Cmnd_Alias", "").replace("LVECTL_CMDS", "").replace("=", "").strip()
                cmd_list = s.split(",")
                for idx1 in range(len(cmd_list)):
                    cmd_list[idx1] = cmd_list[idx1].strip()
                # check command presence
                for sss in ALIAS_LVECTL_CMDS:
                    if sss not in cmd_list:
                        cmd_list.append(sss)
                Clsudo.sudoers_list[idx] = "Cmnd_Alias LVECTL_CMDS = " + ", ".join( ALIAS_LVECTL_CMDS )
                break
        Clsudo._write_contents()
    update_user = staticmethod(update_user)

    def _check_user(user):
        """
        Checks passwd database for username presence
        @param user: string
        """
        try:
            pwd.getpwnam(user)
        except KeyError:
            raise NoSuchUser(user)
    _check_user = staticmethod(_check_user)

    def _get_contents(user):
        """
        Reads file into list of strings
        @param filename: string
        """
        # Clear all status flags
        Clsudo.has_action = False
        Clsudo.has_alias = False
        Clsudo.has_rights = False
        Clsudo.has_selector_alias = False
        Clsudo.has_selector_rights = False
        Clsudo.has_cagefs_alias = False
        Clsudo.has_cagefs_rights = False
        require_tty_pattern = re.compile(
            r'Defaults:\s*%s\s*!requiretty' % (user))

        try:
            i = open(Clsudo.filepath)
            Clsudo.sudoers_list = i.read().splitlines()
            i.close()
            for idx in range(len(Clsudo.sudoers_list)):
                if "Cmnd_Alias LVECTL_CMDS" in Clsudo.sudoers_list[idx]:
                    Clsudo.has_alias = True
                    continue
                if "Cmnd_Alias CAGEFS_CMDS" in Clsudo.sudoers_list[idx]:
                    Clsudo.has_cagefs_alias = True
                    continue
                if ("%s ALL=NOPASSWD: LVECTL_CMDS" % (user,)
                        in Clsudo.sudoers_list[idx]):
                    Clsudo.has_rights = True
                    continue
                if "%s ALL=NOPASSWD: CAGEFS_CMDS" % (user,) in Clsudo.sudoers_list[idx]:
                    Clsudo.has_cagefs_rights = True
                    continue
                if "requiretty" in Clsudo.sudoers_list[idx]:
                    pattern_match = require_tty_pattern.search(
                        Clsudo.sudoers_list[idx])
                    if pattern_match:
                        Clsudo.has_action = True
                    continue
                if "Cmnd_Alias SELECTOR_CMDS" in Clsudo.sudoers_list[idx]:
                    if 'piniset' not in Clsudo.sudoers_list[idx]:
                        Clsudo.sudoers_list[idx] = Clsudo.sudoers_list[idx].replace(
                            '/usr/bin/cl-selector',
                            '/usr/bin/cl-selector, /usr/bin/piniset')
                    if 'lveps' not in Clsudo.sudoers_list[idx]:
                        Clsudo.sudoers_list[idx] = Clsudo.sudoers_list[idx].replace(
                            '/usr/bin/cl-selector, /usr/bin/piniset',
                            '/usr/bin/cl-selector, /usr/bin/piniset, /usr/sbin/lveps')
                    Clsudo.has_selector_alias = True
                    continue
                if ("%s ALL=NOPASSWD: SELECTOR_CMDS" % (user,)
                        in Clsudo.sudoers_list[idx]):
                    Clsudo.has_selector_rights = True
                    continue
        except (IOError, OSError):
            raise UnableToReadFile()
    _get_contents = staticmethod(_get_contents)

    def _write_contents():
        """
        Writes data to temporary file then checks it and rewrites sudoers file
        """
        try:
            fd, temp_path = tempfile.mkstemp(
                prefix=Clsudo.temp_prefix, dir=Clsudo.temp_dir)
            fo = os.fdopen(fd, 'w')
            fo.write('\n'.join(Clsudo.sudoers_list) + '\n')
            fo.close()
            mask = S_IRUSR | S_IRGRP
            os.chmod(temp_path, mask)
            if not Clsudo._is_file_valid(temp_path):
                raise IOError
        except (IOError, OSError):
            try:
                if os.path.exists(temp_path):
                    os.unlink(temp_path)
            except:
                pass
            raise UnableToWriteFile()
        try:
            os.rename(temp_path, Clsudo.filepath)
        except OSError:
            raise UnableToWriteFile()
    _write_contents = staticmethod(_write_contents)

    def _is_file_valid(filename):
        cmd = [
            '/usr/sbin/visudo',
            '-c',
            '-f', filename
        ]
        rv = subprocess.Popen(
            cmd,
            stdin=open('/dev/null'),
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            close_fds=True)
        rt = rv.communicate()
        if rv.returncode != 0:
            return False
        return True
    _is_file_valid = staticmethod(_is_file_valid)
